@@ -21,14 +21,14 @@
 
 #include <sys/param.h>
 
-#if __FreeBSD_version >= 700000
-#error This driver does not support FreeBSD 7.x/-CURRENT!
+#if __FreeBSD_version >= 900000
+#error This driver does not support FreeBSD 9.x/-CURRENT!
 #endif
-#if __FreeBSD_version >= 600000 && __FreeBSD_version < 600034
-#error This driver does not support FreeBSD 6.x/-CURRENT!
+#if __FreeBSD_version >= 800000 && __FreeBSD_version < 800107
+#error This driver requires FreeBSD 8.0 or later!
 #endif
-#if __FreeBSD_version < 503000
-#error This driver requires FreeBSD 5.3 or later!
+#if __FreeBSD_version < 702106
+#error This driver requires FreeBSD 7.3 or later!
 #endif
 
 #include <sys/systm.h>
@@ -51,6 +51,7 @@
 #include <machine/clock.h>
 #include <machine/stdarg.h>
 #include <machine/bus.h>
+#include <machine/specialreg.h>
 
 #include <sys/conf.h>
 #include <sys/rman.h>
@@ -74,18 +75,15 @@
 #include <vm/vm_object.h>
 #include <vm/pmap.h>
 #include <vm/vm_map.h>
+#include <vm/vm_pager.h>
 #include <vm/uma.h>
 
+#if __FreeBSD_version < 800004
 #include <pci/agpvar.h>
-#include <sys/agpio.h>
-
-#if defined(NVCPU_X86_64)
-#define NV_MMAP_TO_VM_OFFSET(_off) ((_off) | 0xfffff00000000000)
-#define NV_VM_TO_MMAP_OFFSET(_off) ((_off) & 0x00000fffffffffff)
 #else
-#define NV_MMAP_TO_VM_OFFSET(_off) ((_off) + VM_MIN_KERNEL_ADDRESS)
-#define NV_VM_TO_MMAP_OFFSET(_off) ((_off) - VM_MIN_KERNEL_ADDRESS)
+#include <dev/agp/agpvar.h>
 #endif
+#include <sys/agpio.h>
 
 #include <sys/smp.h>
 #include <dev/pci/pcireg.h>
@@ -93,33 +91,21 @@
 #include <sys/kdb.h>
 #include <sys/filedesc.h>
 
+#include <sys/priv.h>
 #include <sys/lock.h>
 #include <sys/mutex.h>
 #include <sys/sx.h>
 #include <sys/condvar.h>
-
-#if defined(NVCPU_X86) && defined(PAE)
-#error This driver does not support PAE enabled kernels!
-#endif
+#include <sys/sglist.h>
 
 #define CURTHREAD curthread
 
-/*
- * The resource manager client tracking needs an identifier that uniquely
- * represents a client connection across threads. It needs to be specific
- * to the thread that allocated the client. The Linux struct file pointer
- * is such an identifier. The per-process file descriptor table is enough
- * on FreeBSD, due to its thread-aware reference counting.
- */
-
-#define __TD_FDT(_td) (void *)((_td)->td_proc->p_fd)
-
-#if __FreeBSD_version >= 601100
-#define __NV_ITHREAD() (curthread->td_pflags & TDP_ITHREAD)
-#else
-#define __NV_ITHREAD() (curthread->td_ithd != NULL)
+#if __FreeBSD_version >= 800049
+#define suser(_td) priv_check((_td), PRIV_DRIVER)
 #endif
 
+#define __NV_ITHREAD() (curthread->td_pflags & TDP_ITHREAD)
+
 /*
  * The NVIDIA kernel module's malloc identifier, needed for both tracking
  * and actual allocation/freeing purposes - declared here, but defined in
@@ -128,9 +114,6 @@
 
 MALLOC_DECLARE(M_NVIDIA);
 
-RM_STATUS os_alloc_contig_pages(void **, U032);
-void os_free_contig_pages(void *, U032);
-
 /*
  * Enable/Disable support for FreeBSD's AGP GART driver. Please note that
  * agp.ko may need to be preloaded from loader.conf; this define does not
@@ -155,6 +138,13 @@ void os_free_contig_pages(void *, U032);
 
 #undef NV_SUPPORT_ACPI_PM
 
+/*
+ * Enable/Disable heavy-weight cache-flush logic. By default, the driver
+ * relies on the kernel to perform cache flushes using optimized
+ * routines.
+ */
+
+#undef NV_USE_WBINVD
 
 /*
  * The DMA memory allocation tracking structure. DMA memory alloctions on
@@ -163,14 +153,21 @@ void os_free_contig_pages(void *, U032);
  * a contiguous range of addresses in the AGP aperture, base and size are
  * sufficient to track allocations.
  */
+typedef
+struct nvidia_pte {
+    vm_offset_t virtual_address;
+    uint64_t physical_address;
+} nvidia_pte_t;
 
 typedef
 struct nvidia_alloc {
     SLIST_ENTRY(nvidia_alloc) list;
-    u_int32_t size;
-    vm_offset_t address;
+    vm_memattr_t attr;
+    uint32_t size;
     int alloc_type_contiguous;
-    NvU64 *pte_array;
+    struct nvidia_pte *pte_array;
+    struct sglist *sg_list;
+    vm_object_t object;
 } nvidia_alloc_t;
 
 typedef
@@ -181,9 +178,10 @@ struct nvidia_event {
 
 typedef
 struct nvidia_filep {
-    STAILQ_ENTRY(nvidia_filep) queue;
-    void *fd_table;
-    u_int32_t fd_refcnt;
+    nv_state_t *nv;
+    STAILQ_HEAD(event_queue, nvidia_event) event_queue;
+    struct mtx event_mtx;
+    struct selinfo event_rsel;
 } nvidia_filep_t;
 
 typedef
@@ -191,8 +189,16 @@ struct nvidia_softc {
     device_t dev;
     device_t agp_dev;
 
+    struct sglist *AGP_sg_list;
+    vm_object_t AGP_object;
+
+    struct sglist *UD_sg_list;
+    vm_object_t UD_object;
+
     struct resource *BAR_recs[NV_GPU_NUM_BARS];
     int BAR_rids[NV_GPU_NUM_BARS];
+    struct sglist *BAR_sg_lists[NV_GPU_NUM_BARS];
+    vm_object_t BAR_objects[NV_GPU_NUM_BARS];
 
     struct resource *irq;
     void *irq_ih;
@@ -213,29 +219,21 @@ struct nvidia_softc {
     struct cdev *cdev;
     nv_state_t *nv_state;
 
-    /* queue of OS events */
-    STAILQ_HEAD(event_queue, nvidia_event) event_queue;
-
-    /* queue of fd table references */
-    STAILQ_HEAD(filep_queue, nvidia_filep) filep_queue;
-
     struct sysctl_ctx_list sysctl_ctx;
-    struct selinfo rsel;
 
-    struct callout_handle timer_ch;
+    struct callout timer;
+
+    uint64_t dma_mask;
 
     /* list of allocations */
     SLIST_HEAD(alloc_list, nvidia_alloc) alloc_list;
 
     uint32_t refcnt;
 
-    struct mtx rm_mtx;
     struct sx api_sx;
 
 } nvidia_softc_t;
 
-
-#define CDEV_MAJOR      180
 #define CDEV_CTL_MINOR  255
 
 extern devclass_t nvidia_devclass;
@@ -250,8 +248,21 @@ extern const char *pNVRM_ID;
 #define PCIR_CAP_ID_AGP    0x02
 #define PCIR_CAP_ID_EXP    0x10
 
-#ifndef PCIS_DISPLAY_3D
-#define PCIS_DISPLAY_3D    0x02
+#if !defined(PCIS_DISPLAY_3D)
+#define PCIS_DISPLAY_3D    0x002
+#endif
+#if !defined(PCIM_CMD_INTXDIS)
+#define PCIM_CMD_INTXDIS   0x400
+#endif
+
+#if !defined(PAT_UNCACHEABLE)
+#define PAT_UNCACHEABLE         0x00
+#endif
+#if !defined(PAT_WRITE_COMBINING)
+#define PAT_WRITE_COMBINING     0x01
+#endif
+#if !defined(PAT_WRITE_BACK)
+#define PAT_WRITE_BACK          0x06
 #endif
 
 /*
@@ -273,6 +284,7 @@ extern nv_parm_t nv_parms[];
 
 #define __NV_IOC_TYPE(_cmd) (((_cmd) >> 8) & 0xff)
 #define __NV_IOC_NR(_cmd)   (((_cmd) >> 0) & 0xff)
+#define __NV_IOC_SIZE(_cmd) (((_cmd) >> 16) & 0x1fff)
 
 extern uma_zone_t nvidia_stack_t_zone;
 
@@ -304,15 +316,21 @@ int    nvidia_ctl_detach     (void);
 void   nvidia_pci_check_config_space  (nv_stack_t *, device_t dev, BOOL, BOOL, BOOL);
 int    nvidia_pci_setup_intr          (device_t dev);
 int    nvidia_pci_teardown_intr       (device_t dev);
-U008   nvidia_pci_find_capability     (device_t dev, U008);
+NvU8   nvidia_pci_find_capability     (device_t dev, NvU8);
 
-#define NV_PCI_CHECK_CONFIG_SPACE(_sp,_nv,_cb,_as,_mb)                   \
-    {                                                                    \
-        struct nvidia_softc *__sc;                                       \
-        if (((_nv)->flags & NV_FLAG_CONTROL) == 0) {                     \
-            __sc = (_nv)->os_state;                                      \
-            nvidia_pci_check_config_space(_sp,__sc->dev,_cb,_as,_mb);    \
-        }                                                                \
+#define NV_SHUTDOWN_ADAPTER(sp,nv)                                      \
+    {                                                                   \
+        rm_disable_adapter(sp, nv);                                     \
+        rm_shutdown_adapter(sp, nv);                                    \
+    }
+
+#define NV_PCI_CHECK_CONFIG_SPACE(sp,nv,cb,as,mb)                       \
+    {                                                                   \
+        struct nvidia_softc *__sc;                                      \
+        if (((nv)->flags & NV_FLAG_CONTROL) == 0) {                     \
+            __sc = (nv)->os_state;                                      \
+            nvidia_pci_check_config_space(sp, __sc->dev, cb, as, mb);   \
+        }                                                               \
     }
 
 /* nvidia_subr.c */
@@ -330,15 +348,41 @@ void   nvidia_rc_timer       (void *);
 void   nv_lock_api           (nv_state_t *);
 void   nv_unlock_api         (nv_state_t *);
 
-S032   nv_alloc_contig_pages (nv_state_t *, U032, U032, NvU64 *, void **);
-S032   nv_free_contig_pages  (nv_state_t *, void *);
-S032   nv_alloc_system_pages (nv_state_t *, U032, U032, NvU64 *, void **);
-S032   nv_free_system_pages  (nv_state_t *, void *);
+NvS32  nv_alloc_contig_pages (nv_state_t *, NvU32, NvU32, NvU64 *, void **);
+NvS32  nv_free_contig_pages  (nv_state_t *, void *);
+NvS32  nv_alloc_system_pages (nv_state_t *, NvU32, NvU32, NvU64 *, void **);
+NvS32  nv_free_system_pages  (nv_state_t *, void *);
 
-S032   nv_os_agp_init        (nv_state_t *, void **, U032 *);
-S032   nv_os_agp_teardown    (nv_state_t *);
-S032   nv_alloc_agp_pages    (nv_state_t *, U032, U032, void **);
-S032   nv_free_agp_pages     (nv_state_t *, U032, void *, U032 *);
+NvS32  nv_os_agp_init        (nv_stack_t *sp, nv_state_t *, NvU64 *, NvU64 *);
+NvS32  nv_os_agp_teardown    (nv_stack_t *sp, nv_state_t *);
+NvS32  nv_alloc_agp_pages    (nv_state_t *, NvU32, NvU32, void **);
+NvS32  nv_free_agp_pages     (nv_state_t *, NvU32, void *, NvU32 *);
+
+#define NV_SGLIST_FREE(sgl)             \
+    {                                   \
+        sglist_free(sgl);               \
+        (sgl) = NULL;                   \
+    }
+
+#define NV_VM_OBJECT_DEALLOCATE(obj)    \
+    {                                   \
+        vm_object_deallocate(obj);      \
+        (obj) = NULL;                   \
+    }
+
+#if __FreeBSD_version >= 800000
+#define NV_VM_PAGER_ALLOCATE(tp,hd,sz,pt,of,cr) \
+  vm_pager_allocate(tp, hd, sz, pt, of, cr)
+#else
+#define NV_VM_PAGER_ALLOCATE(tp,hd,sz,pt,of,cr) \
+  vm_pager_allocate(tp, hd, sz, pt, of)
+#endif
+
+#if __FreeBSD_version >= 800000
+#define NV_KDB_ENTER(why,msg) kdb_enter(why, msg)
+#else
+#define NV_KDB_ENTER(why,msg) kdb_enter(msg)
+#endif
 
 /* nvidia_sysctl.c */
 void   nvidia_sysctl_init    (void);
@@ -362,8 +406,6 @@ int    nvidia_sysctl_agp_rate_status  (SYSCTL_HANDLER_ARGS);
 int    nvidia_sysctl_agp_fw_status    (SYSCTL_HANDLER_ARGS);
 int    nvidia_sysctl_agp_sba_status   (SYSCTL_HANDLER_ARGS);
 
-int    nvidia_sysctl_stackwatch     (SYSCTL_HANDLER_ARGS);
-
 void   nv_sysctl_init        (nv_state_t *);
 void   nv_sysctl_exit        (nv_state_t *);
 
@@ -377,14 +419,13 @@ void   nvidia_update_registry (char *);
 
 /* ioctl helpers */
 int    nvidia_get_card_info  (struct nv_ioctl_card_info *);
-int    nvidia_handle_ioctl   (struct cdev *, u_long, caddr_t, int, d_thread_t *);
-
+int    nvidia_handle_ioctl   (nv_state_t *, struct nvidia_filep *, u_long, caddr_t);
 
 /* device helpers */
-int    nvidia_open_ctl       (struct cdev *, d_thread_t *);
-int    nvidia_open_dev       (struct nvidia_softc *, struct cdev *, d_thread_t *);
-int    nvidia_close_ctl      (struct cdev *, d_thread_t *);
-int    nvidia_close_dev      (struct nvidia_softc *, struct cdev *, d_thread_t *);
-int    nvidia_mmap_dev       (struct nvidia_softc *, vm_offset_t, vm_offset_t *);
+int    nvidia_open_ctl       (nv_state_t *, struct nvidia_filep *);
+int    nvidia_open_dev       (nv_state_t *, struct nvidia_filep *);
+int    nvidia_close_ctl      (nv_state_t *, struct nvidia_filep *);
+int    nvidia_close_dev      (nv_state_t *, struct nvidia_filep *);
+int    nvidia_mmap_dev_single(nv_state_t *, vm_ooffset_t *, vm_size_t, vm_object_t *);
 
 #endif /* __NV_FREEBSD_H__ */
