@@ -21,39 +21,61 @@
 #define NV_MAX(_a,_b) ((_a) > (_b) ? (_a) : (_b))
 #endif
 
-/* Nvidia's reserved major device number
- * Set this to 0 to request dynamic major number allocation. */
+/* NVIDIA's reserved major character device number (Linux). */
 #define NV_MAJOR_DEVICE_NUMBER 195
 
 /* most cards in a single system */
-#define NV_MAX_DEVICES 8
+#define NV_MAX_DEVICES 32
 
+/* NOTE: using an ioctl() number > 55 will overflow! */
 #define NV_IOCTL_MAGIC      'F'
 #define NV_IOCTL_BASE       200
 #define NV_ESC_CARD_INFO         (NV_IOCTL_BASE + 0)
 #define NV_ESC_ENV_INFO          (NV_IOCTL_BASE + 2)
-#define NV_ESC_MODULE_RESET      (NV_IOCTL_BASE + 4)
 #define NV_ESC_ALLOC_OS_EVENT    (NV_IOCTL_BASE + 6)
 #define NV_ESC_FREE_OS_EVENT     (NV_IOCTL_BASE + 7)
 #define NV_ESC_STATUS_CODE       (NV_IOCTL_BASE + 9)
 #define NV_ESC_CHECK_VERSION_STR (NV_IOCTL_BASE + 10)
+#define NV_ESC_IOCTL_XFER_CMD    (NV_IOCTL_BASE + 11)
 
-/* NOTE: using an ioctl number > 55 will overflow! */
+#if defined(NV_SUNOS)
+#define NV_PLATFORM_MAX_IOCTL_SIZE  255
+#elif (defined(NV_LINUX) || defined(NV_BSD) || defined(NV_USER_MAP))
+#define NV_PLATFORM_MAX_IOCTL_SIZE  4096
+#else
+#error "The maximum ioctl() argument size is undefined!"
+#endif
+
+/*
+ * #define an absolute maximum used as a sanity check for the
+ * NV_ESC_IOCTL_XFER_CMD ioctl() size argument.
+ */
+#define NV_ABSOLUTE_MAX_IOCTL_SIZE  4096
 
 /*
  * Solaris provides no more than 8 bits for the argument size in
  * the ioctl() command encoding; make sure we don't exceed this
  * limit.
  */
-#if !defined(NV_SUNPRO_C)
-#define __NV_IOWR_ASSERT(type) ((sizeof(type) <= 0xff) ? 1 : -1)
+#define __NV_IOWR_ASSERT(type) ((sizeof(type) <= NV_PLATFORM_MAX_IOCTL_SIZE) ? 1 : -1)
 #define __NV_IOWR(nr, type) ({                                        \
     typedef char __NV_IOWR_TYPE_SIZE_ASSERT[__NV_IOWR_ASSERT(type)];  \
     _IOWR(NV_IOCTL_MAGIC, (nr), type);                                \
 })
-#else
-#define __NV_IOWR(nr, type) (_IOWR(NV_IOCTL_MAGIC, (nr), type))
-#endif
+
+/*
+ * ioctl()'s with parameter structures too large for the
+ * _IOC cmd layout use the nv_ioctl_xfer_t structure
+ * and the NV_ESC_IOCTL_XFER_CMD ioctl() to pass the actual
+ * size and user argument pointer into the RM, which
+ * will then copy it to/from kernel space in separate steps.
+ */
+typedef struct nv_ioctl_xfer
+{
+    NvU32   cmd;
+    NvU32   size;
+    NvP64   ptr  NV_ALIGN_BYTES(8);
+} nv_ioctl_xfer_t;
 
 typedef struct nv_ioctl_card_info
 {
@@ -63,10 +85,10 @@ typedef struct nv_ioctl_card_info
     NvU16    vendor_id;           /* PCI vendor id               */
     NvU16    device_id;
     NvU16    interrupt_line;
-    NvU32    reg_address;         /* register aperture           */
-    NvU32    reg_size;
-    NvU32    fb_address;          /* framebuffer aperture        */
-    NvU32    fb_size;
+    NvU64    reg_address    NV_ALIGN_BYTES(8);
+    NvU64    reg_size       NV_ALIGN_BYTES(8);
+    NvU64    fb_address     NV_ALIGN_BYTES(8);
+    NvU64    fb_size        NV_ALIGN_BYTES(8);
 } nv_ioctl_card_info_t;
 
 #define NV_IOCTL_CARD_INFO_BUS_TYPE_PCI            0x0001
@@ -74,10 +96,6 @@ typedef struct nv_ioctl_card_info
 #define NV_IOCTL_CARD_INFO_BUS_TYPE_PCI_EXPRESS    0x0003
 
 #define NV_IOCTL_CARD_INFO_FLAG_PRESENT       0x0001
-#define NV_IOCTL_CARD_INFO_FLAG_NEED_MSYNC    0x0002
-
-#define NV_IOCTL_CARD_INFO     \
-    __NV_IOWR(NV_ESC_CARD_INFO, NvU8[NV_MAX_DEVICES*sizeof(nv_ioctl_card_info_t)])
 
 #define SIM_ENV_GPU       0
 #define SIM_ENV_IKOS      1
@@ -88,12 +106,9 @@ typedef struct nv_ioctl_card_info
 
 typedef struct nv_ioctl_env_info
 {
-    NvU32 sim_env;
     NvU32 pat_supported;
 } nv_ioctl_env_info_t;
 
-#define NV_IOCTL_ENV_INFO  __NV_IOWR(NV_ESC_ENV_INFO, nv_ioctl_env_info_t)
-
 /* old rm api check
  *
  * this used to be used to verify client/rm interaction both ways by
@@ -116,9 +131,6 @@ typedef struct nv_ioctl_rm_api_old_version
 #define NV_RM_API_OLD_VERSION_MAGIC_OVERRIDE_REQ    (NV_RM_API_OLD_VERSION_MAGIC_REQ ^ '2')
 #define NV_RM_API_OLD_VERSION_MAGIC_IGNORE           0xffffffff
 
-/* debug tool to force module count to 0 */
-#define NV_IOCTL_MODULE_RESET  _IO(NV_IOCTL_MAGIC, NV_ESC_MODULE_RESET)
-
 /* alloc event */
 typedef struct nv_ioctl_alloc_os_event
 {
@@ -129,8 +141,6 @@ typedef struct nv_ioctl_alloc_os_event
     NvU32 Status;
 } nv_ioctl_alloc_os_event_t;
 
-#define NV_IOCTL_ALLOC_OS_EVENT __NV_IOWR(NV_ESC_ALLOC_OS_EVENT, nv_ioctl_alloc_os_event_t)
-
 /* free event */
 typedef struct nv_ioctl_free_os_event
 {
@@ -140,8 +150,6 @@ typedef struct nv_ioctl_free_os_event
     NvU32 Status;
 } nv_ioctl_free_os_event_t;
 
-#define NV_IOCTL_FREE_OS_EVENT __NV_IOWR(NV_ESC_FREE_OS_EVENT, nv_ioctl_free_os_event_t)
-
 /* status code */
 typedef struct nv_ioctl_status_code
 {
@@ -150,9 +158,6 @@ typedef struct nv_ioctl_status_code
     NvU32 status;
 } nv_ioctl_status_code_t;
 
-#define NV_IOCTL_STATUS_CODE __NV_IOWR(NV_ESC_STATUS_CODE, nv_ioctl_status_code_t)
-
-
 /* check version string */
 #define NV_RM_API_VERSION_STRING_LENGTH 64
 
@@ -170,8 +175,6 @@ typedef struct nv_ioctl_rm_api_version
 #define NV_RM_API_VERSION_REPLY_UNRECOGNIZED 0
 #define NV_RM_API_VERSION_REPLY_RECOGNIZED   1
 
-#define NV_IOCTL_CHECK_VERSION_STR __NV_IOWR(NV_ESC_CHECK_VERSION_STR, nv_ioctl_rm_api_version_t)
-
 #ifdef NVRM
 
 /*
@@ -180,9 +183,9 @@ typedef struct nv_ioctl_rm_api_version
 
 typedef union
 {
-    volatile V008 Reg008[1];
-    volatile V016 Reg016[1];
-    volatile V032 Reg032[1];
+    volatile NvV8 Reg008[1];
+    volatile NvV16 Reg016[1];
+    volatile NvV32 Reg032[1];
 } nv_hwreg_t, * nv_phwreg_t;
 
 
@@ -201,8 +204,8 @@ typedef union
 
 typedef struct
 {
-    NvU32 address;
-    NvU32 size;
+    NvU64 address;
+    NvU64 size;
     NvU32 offset;
     NvU32 *map;
     nv_phwreg_t map_u;
@@ -224,9 +227,9 @@ typedef struct
 
 typedef struct nv_stack_s
 {
-    U032  size;
+    NvU32 size;
     void *top;
-    U008  stack[NV_STACK_SIZE-16] __attribute__ ((aligned(16)));
+    NvU8  stack[NV_STACK_SIZE-16] __attribute__ ((aligned(16)));
 } nv_stack_t;
 
 /*
@@ -248,7 +251,7 @@ typedef struct nv_event_s
 typedef struct nv_kern_mapping_s
 {
     void  *addr;
-    NvU32 size;
+    NvU64 size;
     struct nv_kern_mapping_s *next;
 } nv_kern_mapping_t;
 
@@ -265,42 +268,51 @@ typedef struct
     int    flags;
 
     /* PCI config info */
-    U016 bus;
-    U016 slot;
-    U016 vendor_id;
-    U016 device_id;
+    NvU16 bus;
+    NvU16 slot;
+    NvU16 vendor_id;
+    NvU16 device_id;
+    NvU16 subsystem_id;
     void *handle;
 
-    U032 pci_cfg_space[16];
+    NvU32 pci_cfg_space[16];
 
     /* physical characteristics */
     nv_aperture_t bars[NV_GPU_NUM_BARS];
     nv_aperture_t *regs;
-    nv_aperture_t *fb;
-    nv_aperture_t agp, bc;
+    nv_aperture_t *fb, ud;
+    nv_aperture_t agp;
 
     NvU32  interrupt_line;
 
-    U032 agp_config;
-    U032 agp_status;
-
-    /* save/restore eflags register */
-    unsigned long saved_eflags;
+    NvU32 agp_config;
+    NvU32 agp_status;
 
     NvU32 primary_vga;
 
-    U032 sim_env;
+    NvU32 sim_env;
 
-    U032 rc_timer_enabled;
+    NvU32 rc_timer_enabled;
 
     /* list of events allocated for this device */
     nv_event_t *event_list;
 
     nv_kern_mapping_t *kern_mappings;
 
-    U032 acpi_initialized;
 } nv_state_t;
 
+typedef struct
+{
+    /* PCI config info */
+    NvU16 bus;
+    NvU16 slot;
+    NvU16 vendor_id;
+    NvU16 device_id;
+    void *handle;
+    nv_aperture_t bar0;
+    nv_aperture_t *regs;
+} nv_smu_state_t;
+
 /*
  * flags
  */
@@ -310,6 +322,12 @@ typedef struct
 #define NV_FLAG_CONTROL         0x0004
 #define NV_FLAG_MAP_REGS_EARLY  0x0008
 #define NV_FLAG_USE_BAR0_CFG    0x0010
+#define NV_FLAG_USES_MSI        0x0020
+#define NV_FLAG_S4_AVAILABLE    0x0040
+#define NV_FLAG_PASSTHRU        0x0080
+#define NV_FLAG_GVI_IN_SUSPEND  0x0100
+#define NV_FLAG_GVI             0x0200
+#define NV_FLAG_GVI_INTR_EN     0x0400
 
 #define NV_PM_APM_SUSPEND       0x0001
 #define NV_PM_APM_RESUME        0x0002
@@ -329,11 +347,56 @@ typedef struct
 
 #define NV_PRIMARY_VGA(nv)      ((nv)->primary_vga)
 
-/*
- * This acpi event will be generated when the user presses a hotkey to switch
- * display output from LCD to CRT.
+#define NV_IS_GVI_DEVICE(nv) ((nv)->flags & NV_FLAG_GVI)
+
+/*                                                                                    
+ * The ACPI specification defines IDs for various ACPI video                          
+ * extension events like display switch events, AC/battery                            
+ * events, docking events, etc..                                                      
+ * Whenever an ACPI event is received by the corresponding                            
+ * event handler installed within the core NVIDIA driver, the                         
+ * code can verify the event ID before processing it.                                 
  */
-#define NV_SYSTEM_ACPI_DISPLAY_DEVICE_CYCLE      0x80 
+#define ACPI_DISPLAY_DEVICE_CHANGE_EVENT      0x80 
+#define NVIF_NOTIFY_DISPLAY_DETECT           0xCB
+#define NVIF_DISPLAY_DEVICE_CHANGE_EVENT     NVIF_NOTIFY_DISPLAY_DETECT 
+/*                                                                                    
+ * NVIDIA ACPI event IDs to be passed into the core NVIDIA                            
+ * driver for various events like display switch events,                              
+ * AC/battery events, docking events, etc..                                           
+ */                                                                                   
+#define NV_SYSTEM_ACPI_DISPLAY_SWITCH_EVENT  0x8001                                   
+#define NV_SYSTEM_ACPI_BATTERY_POWER_EVENT   0x8002                                   
+#define NV_SYSTEM_ACPI_DOCK_EVENT            0x8003                                   
+
+/*
+ * Status bit definitions for display switch hotkey events.
+ */
+#define NV_HOTKEY_STATUS_DISPLAY_ENABLE_LCD 0x01
+#define NV_HOTKEY_STATUS_DISPLAY_ENABLE_CRT 0x02
+#define NV_HOTKEY_STATUS_DISPLAY_ENABLE_TV  0x04
+#define NV_HOTKEY_STATUS_DISPLAY_ENABLE_DFP 0x08
+
+/*                                                                                    
+ * NVIDIA ACPI sub-event IDs (event types) to be passed into                          
+ * to core NVIDIA driver for ACPI events.                                             
+ */                                                                                   
+#define NV_SYSTEM_ACPI_EVENT_VALUE_DISPLAY_SWITCH_DEFAULT    0                        
+#define NV_SYSTEM_ACPI_EVENT_VALUE_POWER_EVENT_AC            0                        
+#define NV_SYSTEM_ACPI_EVENT_VALUE_POWER_EVENT_BATTERY       1                        
+#define NV_SYSTEM_ACPI_EVENT_VALUE_DOCK_EVENT_UNDOCKED       0                        
+#define NV_SYSTEM_ACPI_EVENT_VALUE_DOCK_EVENT_DOCKED         1                        
+
+#define NV_ACPI_NVIF_HANDLE_PRESENT 0x01
+#define NV_ACPI_DSM_HANDLE_PRESENT  0x02
+#define NV_ACPI_WMMX_HANDLE_PRESENT 0x04
+#define NV_ACPI_MXMI_HANDLE_PRESENT 0x08
+#define NV_ACPI_MXMS_HANDLE_PRESENT 0x10
+
+#define NV_ACPI_METHOD_NVIF     0x01
+#define NV_ACPI_METHOD_WMMX     0x02
+#define NV_ACPI_METHOD_MXMI     0x03
+#define NV_ACPI_METHOD_MXMS     0x04
 
 /*
 ** where we hide our nv_state_t * ...
@@ -351,10 +414,10 @@ typedef struct
              (((offset) >= (nv)->fb->address) &&                               \
              (((offset) + ((length)-1)) <= (nv)->fb->address + ((nv)->fb->size-1)))
 
-#define IS_BC_OFFSET(nv, offset, length)                                       \
-             (((nv)->bc.address != 0) && ((nv)->bc.size != 0) &&               \
-             ((offset) >= (nv)->bc.address) &&                                 \
-             (((offset) + ((length)-1)) <= (nv)->bc.address + ((nv)->bc.size-1)))
+#define IS_UD_OFFSET(nv, offset, length)                                       \
+             (((nv)->ud.address != 0) && ((nv)->ud.size != 0) &&               \
+             ((offset) >= (nv)->ud.address) &&                                 \
+             (((offset) + ((length)-1)) <= (nv)->ud.address + ((nv)->ud.size-1)))
 
 #define IS_AGP_OFFSET(nv, offset, length)                                      \
              ((NV_AGP_ENABLED(nv)) &&                                          \
@@ -445,39 +508,52 @@ typedef struct
 #endif /* !defined(NV_API_CALL) */
 
 
-NvU64  NV_API_CALL  nv_dma_to_mmap_token         (nv_state_t *, NvU64);
-
-void*  NV_API_CALL  nv_alloc_kernel_mapping      (nv_state_t *, NvU64, U032, void **);
-S032   NV_API_CALL  nv_free_kernel_mapping       (nv_state_t *, void *, void *);
+void*  NV_API_CALL  nv_alloc_kernel_mapping      (nv_state_t *, NvU64, NvU32, void **);
+NvS32  NV_API_CALL  nv_free_kernel_mapping       (nv_state_t *, void *, void *, BOOL);
 
 NvU64  NV_API_CALL  nv_get_kern_phys_address     (NvU64);
 NvU64  NV_API_CALL  nv_get_user_phys_address     (NvU64);
-void*  NV_API_CALL  nv_get_adapter_state         (U016, U016);
+void*  NV_API_CALL  nv_get_adapter_state         (NvU16, NvU16);
 
-void   NV_API_CALL  nv_lock_rm                   (nv_state_t *);
-void   NV_API_CALL  nv_unlock_rm                 (nv_state_t *);
+void   NV_API_CALL  nv_set_dma_address_size      (nv_state_t *, NvU32 );
 
-void   NV_API_CALL  nv_set_dma_address_size      (nv_state_t *, U032 );
+RM_STATUS  NV_API_CALL  nv_alias_pages           (nv_state_t *, NvU32, NvU32, NvU32, NvU64, NvU64 *, void **);
+RM_STATUS  NV_API_CALL  nv_alloc_pages           (nv_state_t *, NvU32, NvU32, NvU32, NvU32, NvU64 *, void **);
+RM_STATUS  NV_API_CALL  nv_free_pages            (nv_state_t *, NvU32, NvU32, NvU32, NvU32, void *);
+RM_STATUS  NV_API_CALL  nv_guest_pfn_list        (nv_state_t *, unsigned int, unsigned int, unsigned int, unsigned int*);
 
-RM_STATUS  NV_API_CALL  nv_alloc_pages           (nv_state_t *, U032, U032, U032, U032, NvU64 *, void **);
-RM_STATUS  NV_API_CALL  nv_free_pages            (nv_state_t *, U032, U032, U032, U032, void *);
-
-RM_STATUS  NV_API_CALL  nv_agp_init              (nv_state_t *, void **, void *, U032);
+RM_STATUS  NV_API_CALL  nv_agp_init              (nv_state_t *, NvU64 *, NvU64 *, NvU32);
 RM_STATUS  NV_API_CALL  nv_agp_teardown          (nv_state_t *);
 
-S032   NV_API_CALL  nv_start_rc_timer            (nv_state_t *);
-S032   NV_API_CALL  nv_stop_rc_timer             (nv_state_t *);
+NvS32  NV_API_CALL  nv_start_rc_timer            (nv_state_t *);
+NvS32  NV_API_CALL  nv_stop_rc_timer             (nv_state_t *);
 
-void   NV_API_CALL  nv_post_event                (nv_state_t *, nv_event_t *, U032, U032);
-S032   NV_API_CALL  nv_get_event                 (nv_state_t *, void *, nv_event_t *, U032 *);
+void   NV_API_CALL  nv_post_event                (nv_state_t *, nv_event_t *, NvU32, NvU32);
+NvS32  NV_API_CALL  nv_get_event                 (nv_state_t *, void *, nv_event_t *, NvU32 *);
 
-S032   NV_API_CALL  nv_no_incoherent_mappings    (void);
+NvS32  NV_API_CALL  nv_no_incoherent_mappings    (void);
 
 void   NV_API_CALL  nv_verify_pci_config         (nv_state_t *, BOOL);
 
-void*  NV_API_CALL  nv_i2c_add_adapter           (nv_state_t *, U032);
+void*  NV_API_CALL  nv_i2c_add_adapter           (nv_state_t *, NvU32);
 BOOL   NV_API_CALL  nv_i2c_del_adapter           (nv_state_t *, void *);
 
+void   NV_API_CALL  nv_acpi_methods_init         (NvU32 *);
+void   NV_API_CALL  nv_acpi_methods_uninit       (void);
+
+RM_STATUS  NV_API_CALL  nv_acpi_method           (NvU32, NvU32, NvU32, void *, NvU16, NvU32 *, void *, NvU16 *);
+RM_STATUS  NV_API_CALL  nv_acpi_dsm_method       (NvU8 *, NvU32, NvU32, void *, NvU16, NvU32 *, void *, NvU16 *);
+void*  NV_API_CALL  nv_get_smu_state             (void); 
+
+#ifdef NV_USER_MAP
+#include "nv-map.h"
+#else /* NV_USER_MAP */
+#define NV_MAP_USER_SPACE(status, nv, va, sz, pr, hclient, hparent, hobject)  do { } while (0)
+#define NV_UNMAP_USER_SPACE(nv, va)  do { } while (0)
+#define NV_UNMAP_OBJECT_USER_SPACE(nv, hclient, hparent, hobject)  do { } while (0)
+#define NV_RELEASE_USERMAP_LIST(nvl) do { } while (0)
+#endif /* NV_USER_MAP */
+
 /*
  * ---------------------------------------------------------------------------
  *
@@ -494,47 +570,47 @@ BOOL       NV_API_CALL  rm_init_adapter          (nv_stack_t *, nv_state_t *);
 BOOL       NV_API_CALL  rm_disable_adapter       (nv_stack_t *, nv_state_t *);
 BOOL       NV_API_CALL  rm_shutdown_adapter      (nv_stack_t *, nv_state_t *);
 void       NV_API_CALL  rm_set_interrupts        (nv_stack_t *, BOOL);
-BOOL       NV_API_CALL  rm_ioctl                 (nv_stack_t *, nv_state_t *, void *, U032, void *);
-BOOL       NV_API_CALL  rm_isr                   (nv_stack_t *, nv_state_t *, U032 *);
+BOOL       NV_API_CALL  rm_ioctl                 (nv_stack_t *, nv_state_t *, void *, NvU32, void *);
+BOOL       NV_API_CALL  rm_isr                   (nv_stack_t *, nv_state_t *, NvU32 *);
 void       NV_API_CALL  rm_isr_bh                (nv_stack_t *, nv_state_t *);
-RM_STATUS  NV_API_CALL  rm_power_management      (nv_stack_t *, nv_state_t *, U032, U032);
+RM_STATUS  NV_API_CALL  rm_power_management      (nv_stack_t *, nv_state_t *, NvU32, NvU32);
 RM_STATUS  NV_API_CALL  rm_save_low_res_mode     (nv_stack_t *, nv_state_t *);
-U032       NV_API_CALL  rm_get_vbios_version     (nv_stack_t *, nv_state_t *, U032 *, U032 *, U032 *, U032 *, U032 *);
+NvU32      NV_API_CALL  rm_get_vbios_version     (nv_stack_t *, nv_state_t *, NvU32 *, NvU32 *, NvU32 *, NvU32 *, NvU32 *);
 void       NV_API_CALL  rm_free_unused_clients   (nv_stack_t *, nv_state_t *, void *);
 
 void       NV_API_CALL  rm_update_agp_config     (nv_stack_t *, nv_state_t *);
-RM_STATUS  NV_API_CALL  rm_init_agp              (nv_stack_t *, nv_state_t *);
+RM_STATUS  NV_API_CALL  rm_init_agp              (nv_stack_t *, nv_state_t *, NvU64 *, NvU64 *);
 RM_STATUS  NV_API_CALL  rm_teardown_agp          (nv_stack_t *, nv_state_t *);
 
-RM_STATUS  NV_API_CALL  rm_alloc_agp_pages       (nv_stack_t *, nv_state_t *, U032, void **, U032 *);
+RM_STATUS  NV_API_CALL  rm_alloc_agp_pages       (nv_stack_t *, nv_state_t *, NvU32, void **, NvU32 *);
 RM_STATUS  NV_API_CALL  rm_free_agp_pages        (nv_stack_t *, nv_state_t *, void *);
 
-RM_STATUS  NV_API_CALL  rm_alloc_agp_bitmap      (nv_stack_t *, nv_state_t *, U032, U032 *);
-RM_STATUS  NV_API_CALL  rm_free_agp_bitmap       (nv_stack_t *, nv_state_t *, U032, U032);
+RM_STATUS  NV_API_CALL  rm_alloc_agp_bitmap      (nv_stack_t *, nv_state_t *, NvU32, NvU32 *);
+RM_STATUS  NV_API_CALL  rm_free_agp_bitmap       (nv_stack_t *, nv_state_t *, NvU32, NvU32);
 RM_STATUS  NV_API_CALL  rm_set_agp_bitmap        (nv_stack_t *, nv_state_t *, void *);
 RM_STATUS  NV_API_CALL  rm_clear_agp_bitmap      (nv_stack_t *, nv_state_t *, void **);
 
-RM_STATUS  NV_API_CALL  rm_read_registry_dword   (nv_stack_t *, nv_state_t *, U008 *, U008 *, U032 *);
-RM_STATUS  NV_API_CALL  rm_write_registry_dword  (nv_stack_t *, nv_state_t *, U008 *, U008 *, U032);
-RM_STATUS  NV_API_CALL  rm_read_registry_binary  (nv_stack_t *, nv_state_t *, U008 *, U008 *, U008 *, U032 *);
-RM_STATUS  NV_API_CALL  rm_write_registry_binary (nv_stack_t *, nv_state_t *, U008 *, U008 *, U008 *, U032);
+RM_STATUS  NV_API_CALL  rm_read_registry_dword   (nv_stack_t *, nv_state_t *, NvU8 *, NvU8 *, NvU32 *);
+RM_STATUS  NV_API_CALL  rm_write_registry_dword  (nv_stack_t *, nv_state_t *, NvU8 *, NvU8 *, NvU32);
+RM_STATUS  NV_API_CALL  rm_read_registry_binary  (nv_stack_t *, nv_state_t *, NvU8 *, NvU8 *, NvU8 *, NvU32 *);
+RM_STATUS  NV_API_CALL  rm_write_registry_binary (nv_stack_t *, nv_state_t *, NvU8 *, NvU8 *, NvU8 *, NvU32);
 
 RM_STATUS  NV_API_CALL  rm_run_rc_callback       (nv_stack_t *, nv_state_t *);
-RM_STATUS  NV_API_CALL  rm_get_device_name       (nv_stack_t *, nv_state_t *, U032, U032, U008*);
+RM_STATUS  NV_API_CALL  rm_get_device_name       (nv_stack_t *, nv_state_t *, NvU32, NvU32, NvU8*);
 
-RM_STATUS  NV_API_CALL  rm_validate_pfn_range    (nv_stack_t *, nv_state_t *, void *, U032, U032, BOOL);
 NvU64      NV_API_CALL  nv_rdtsc                 (void);
 
 void       NV_API_CALL  rm_register_compatible_ioctls   (nv_stack_t *);
 void       NV_API_CALL  rm_unregister_compatible_ioctls (nv_stack_t *);
 
-BOOL       NV_API_CALL  rm_is_legacy_device      (nv_stack_t *, U016, BOOL);
+BOOL       NV_API_CALL  rm_is_legacy_device      (nv_stack_t *, NvU16, BOOL);
+BOOL       NV_API_CALL  rm_is_supported_device   (nv_stack_t *, nv_state_t *);
 
-RM_STATUS  NV_API_CALL  rm_i2c_read_buffer        (nv_stack_t *, nv_state_t *, void *, U008, U032, U008 *);
-RM_STATUS  NV_API_CALL  rm_i2c_write_buffer       (nv_stack_t *, nv_state_t *, void *, U008, U032, U008 *);
-RM_STATUS  NV_API_CALL  rm_i2c_smbus_write_quick  (nv_stack_t *, nv_state_t *, void *, U008, U008);
-RM_STATUS  NV_API_CALL  rm_i2c_smbus_read_buffer  (nv_stack_t *, nv_state_t *, void *, U008, U008, U032, U008 *);
-RM_STATUS  NV_API_CALL  rm_i2c_smbus_write_buffer (nv_stack_t *, nv_state_t *, void *, U008, U008, U032, U008 *);
+RM_STATUS  NV_API_CALL  rm_i2c_read_buffer        (nv_stack_t *, nv_state_t *, void *, NvU8, NvU32, NvU8 *);
+RM_STATUS  NV_API_CALL  rm_i2c_write_buffer       (nv_stack_t *, nv_state_t *, void *, NvU8, NvU32, NvU8 *);
+RM_STATUS  NV_API_CALL  rm_i2c_smbus_write_quick  (nv_stack_t *, nv_state_t *, void *, NvU8, NvU8);
+RM_STATUS  NV_API_CALL  rm_i2c_smbus_read_buffer  (nv_stack_t *, nv_state_t *, void *, NvU8, NvU8, NvU32, NvU8 *);
+RM_STATUS  NV_API_CALL  rm_i2c_smbus_write_buffer (nv_stack_t *, nv_state_t *, void *, NvU8, NvU8, NvU32, NvU8 *);
 
 void       NV_API_CALL  rm_check_pci_config_space (nv_stack_t *, nv_state_t *nv, BOOL, BOOL, BOOL);
 
@@ -542,7 +618,11 @@ RM_STATUS  NV_API_CALL  rm_i2c_remove_adapters    (nv_stack_t *, nv_state_t *);
 
 RM_STATUS  NV_API_CALL  rm_perform_version_check  (nv_stack_t *, nv_ioctl_rm_api_version_t *);
 
-RM_STATUS  NV_API_CALL  rm_system_event           (nv_stack_t *, U032);
+RM_STATUS  NV_API_CALL  rm_system_event           (nv_stack_t *, NvU32, NvU32);
+RM_STATUS  NV_API_CALL  rm_init_smu               (nv_stack_t *, nv_smu_state_t *);
+RM_STATUS  NV_API_CALL  rm_shutdown_smu           (nv_stack_t *, nv_smu_state_t *);
+RM_STATUS  NV_API_CALL  rm_suspend_smu            (nv_stack_t *);
+RM_STATUS  NV_API_CALL  rm_resume_smu             (nv_stack_t *);
 
 #define rm_disable_interrupts(sp) rm_set_interrupts((sp),FALSE)
 #define rm_enable_interrupts(sp)  rm_set_interrupts((sp),TRUE)
@@ -551,34 +631,6 @@ RM_STATUS  NV_API_CALL  rm_system_event           (nv_stack_t *, U032);
 
 #endif /* NVRM */
 
-#ifdef NV_STACKWATCH
-#define CALL_STK_MAX_DEPTH      64
-#define TOP_USERS_NUM           10
-
-struct ftn_rec   {
-    void            *ftn;
-    unsigned long   use;
-};
-
-struct callstk_rec  {
-    struct ftn_rec  callstk[CALL_STK_MAX_DEPTH];
-    int             depth;
-    unsigned long   total_use;
-    unsigned long   signature;
-};
-
-struct stk_stats    {
-    void                *sst_ref_addr;
-    struct ftn_rec      sst_top_users[TOP_USERS_NUM];   // Top stack user functions
-    struct callstk_rec  sst_top_stacks[TOP_USERS_NUM];  // Top stack user call stacks
-};
-
-extern struct stk_stats stk_stats;
-
-void    nv_stackwatch_freeze(void);
-void    nv_stackwatch_reset(void);
-#endif  // NV_STACKWATCH
-
 static inline int nv_count_bits(NvU64 word)
 {
     NvU64 bits;
