@@ -25,23 +25,6 @@ MALLOC_DEFINE(M_NVIDIA, "nvidia", "NVIDIA memory allocations");
 
 #define MAX_ERROR_STRING    256
 
-RM_STATUS NV_API_CALL os_alloc_contig_pages(
-    void **address,
-    U032 size
-)
-{
-    *address = contigmalloc(size, M_NVIDIA, 0, 0, ~0, PAGE_SIZE, 0);
-    return *address ? RM_OK : RM_ERROR;
-}
-
-void NV_API_CALL os_free_contig_pages(
-    void *address,
-    U032 size
-)
-{
-    contigfree(address, size, M_NVIDIA);
-}
-
 /*
  * The core resource manager's favorite source of memory, this routine is
  * called from different contexts, including ISRs. This means that it can
@@ -129,21 +112,12 @@ U032 NV_API_CALL os_get_current_process(void)
     return curproc->p_pid;
 }
 
-RM_STATUS NV_API_CALL os_kill_process(
-    U032 pid,
-    U032 sig
-)
+RM_STATUS NV_API_CALL os_get_current_thread(NvU64 *threadId)
 {
-    struct proc *p = pfind(pid);
-    if (!p) {
-        /*
-         * No process with the specified PID was found running. We notify
-         * the caller, but take no other action.
-         */
-        return RM_ERR_OPERATING_SYSTEM;
-    }
-
-    psignal(p, sig);
+    if (__NV_ITHREAD())
+        *threadId = 0;
+    else
+        *threadId = (NvU64) CURTHREAD->td_tid;
 
     return RM_OK;
 }
@@ -169,7 +143,6 @@ BOOL NV_API_CALL os_is_administrator(PHWINFO pDev)
 }
 
 U008 NV_API_CALL os_io_read_byte(
-    PHWINFO pDev,
     U032 address
 )
 {
@@ -178,7 +151,6 @@ U008 NV_API_CALL os_io_read_byte(
 }
 
 void NV_API_CALL os_io_write_byte(
-    PHWINFO pDev,
     U032 address,
     U008 value
 )
@@ -188,7 +160,6 @@ void NV_API_CALL os_io_write_byte(
 }
 
 U016 NV_API_CALL os_io_read_word(
-    PHWINFO pDev,
     U032 address
 )
 {
@@ -197,7 +168,6 @@ U016 NV_API_CALL os_io_read_word(
 }
 
 void NV_API_CALL os_io_write_word(
-    PHWINFO pDev,
     U032 address,
     U016 value
 )
@@ -207,7 +177,6 @@ void NV_API_CALL os_io_write_word(
 }
 
 U032 NV_API_CALL os_io_read_dword(
-    PHWINFO pDev,
     U032 address
 )
 {
@@ -216,7 +185,6 @@ U032 NV_API_CALL os_io_read_dword(
 }
 
 void NV_API_CALL os_io_write_dword(
-    PHWINFO pDev,
     U032 address,
     U032 value
 )
@@ -225,29 +193,16 @@ void NV_API_CALL os_io_write_dword(
     outl(address, value);
 }
 
-/*
- * XXX On FreeBSD/amd64, pmap_mapdev() returns an address range from the
- * kernel's direct mapping of the first 4+GB; this direct mapping uses
- * 2MB pages and may be cached. Ideally, we'd want to modify it for as long
- * as we need uncached access to the I/O memory in question, but for now
- * we'll have to create a private, uncached mapping to be safe.
- *
- * XXX This introduces aliasing concerns, but it's better than using the
- * cached direct mapping.
- */
-
 void* NV_API_CALL os_map_kernel_space(
     NvU64 start,
     NvU64 size,
     U032 mode
 )
 {
-    void *vm;
-    U032 cache_bits = (mode != NV_MEMORY_CACHED) ? PG_N : 0;
-    vm_offset_t tva, va;
+    int map_mode;
 
 #if defined(NVCPU_X86) && !defined(PAE)
-    if (start > 0xffffffff)
+    if (start > 0xffffffffULL)
         return NULL;
 #endif
 
@@ -256,19 +211,24 @@ void* NV_API_CALL os_map_kernel_space(
 
     size = NV_ALIGN_UP(size, PAGE_SIZE);
 
-    va = kmem_alloc_nofault(kernel_map, size);
-    vm = (void *)va;
-
-    if (vm != NULL) {
-        for (tva = va; tva < (va + size); tva += PAGE_SIZE) {
-            pt_entry_t *ptep = vtopte(tva);
-            pte_store(ptep, start | PG_RW | PG_V | PG_G | cache_bits);
-            start += PAGE_SIZE;
-        }
-        pmap_invalidate_range(kernel_pmap, va, tva);
+    switch (mode) {
+        case NV_MEMORY_CACHED:
+            map_mode = PAT_WRITE_BACK;
+            break;
+        case NV_MEMORY_WRITECOMBINED:
+            map_mode = PAT_WRITE_COMBINING;
+            break;
+        case NV_MEMORY_UNCACHED:
+        case NV_MEMORY_DEFAULT:
+            map_mode = PAT_UNCACHEABLE;
+            break;
+        default:
+            nv_printf(NV_DBG_ERRORS,
+                      "NVRM: unknown mode in os_map_kernel_space()\n");
+            return NULL;
     }
 
-    return vm;
+    return pmap_mapdev_attr(start, size, map_mode);
 }
 
 void NV_API_CALL os_unmap_kernel_space(
@@ -276,44 +236,12 @@ void NV_API_CALL os_unmap_kernel_space(
     NvU64 size
 )
 {
-    vm_offset_t tva, va;
-
-    size = NV_ALIGN_UP(size, PAGE_SIZE);
-
-    va = (vm_offset_t) address;
-    if (va != 0) {
-        for (tva = va; tva < (va + size); tva += PAGE_SIZE)
-            pte_clear(vtopte(tva));
-        pmap_invalidate_range(kernel_pmap, va, tva);
-        kmem_free(kernel_map, va, size);
-    }
-}
-
-void* NV_API_CALL os_map_kernel_space_high(
-    U032 pfn,
-    U032 size
-)
-{
-    U032 start;
-    if (!(pfn & ~0xfffff)) {
-        start = pfn << PAGE_SHIFT;
-        return os_map_kernel_space(start, size, NV_MEMORY_CACHED);
-    }
-    return NULL;
-}
-
-void NV_API_CALL os_unmap_kernel_space_high(
-    void *addr,
-    U032 pfn,
-    U032 size
-)
-{
-    os_unmap_kernel_space(addr, size);
+    pmap_unmapdev((vm_offset_t)address, size);
 }
 
 RM_STATUS NV_API_CALL os_set_mem_range(
-    U032 start,
-    U032 size,
+    NvU64 start,
+    NvU64 size,
     U032 mode
 )
 {
@@ -334,8 +262,8 @@ RM_STATUS NV_API_CALL os_set_mem_range(
 }
 
 RM_STATUS NV_API_CALL os_unset_mem_range(
-    U032 start,
-    U032 size
+    NvU64 start,
+    NvU64 size
 )
 {
     int arg;
@@ -378,24 +306,26 @@ void NV_API_CALL os_dbg_init(void)
     NV_UMA_ZONE_FREE_STACK(sp);
 }
 
+RM_STATUS NV_API_CALL os_schedule(void)
+{
+    return RM_ERR_NOT_SUPPORTED;
+}
+
 void NV_API_CALL os_dbg_set_level(U032 new_debuglevel)
 {
     cur_debuglevel = new_debuglevel;
 }
 
-void NV_API_CALL NV_STACKWATCH_CALLBACK os_dbg_breakpoint(void)
+void NV_API_CALL os_dbg_breakpoint(void)
 {
 #ifdef DEBUG
-    /* FreeBSD 5.x kdb */
-    kdb_enter("os_dbg_breakpoint()");
+    NV_KDB_ENTER("breakpoint", "DEBUG breakpoint");
 #endif
 }
 
 void NV_API_CALL out_string(const char *message)
 {
-#if defined(DEBUG) || defined(QA_BUILD)
     printf("%s", message);
-#endif
 }
 
 static char nv_error_string[MAX_ERROR_STRING];
@@ -507,30 +437,22 @@ void* NV_API_CALL os_mem_set(
     return memset(b, c, length);
 }
 
-S032 NV_API_CALL os_string_compare(
-    const U008 *s1,
-    const U008 *s2
-)
-{
-    return strcmp(s1, s2);
-}
-
-U008* NV_API_CALL os_string_copy(
-    U008 *dst,
-    const U008 *src
+char* NV_API_CALL os_string_copy(
+    char *dst,
+    const char *src
 )
 {
     return strcpy(dst, src);
 }
 
-U032 NV_API_CALL os_string_length(const U008* s)
+U032 NV_API_CALL os_string_length(const char* s)
 {
     return strlen(s);
 }
 
 RM_STATUS NV_API_CALL os_strncpy_from_user(
-    U008 *dst,
-    const U008 *src,
+    char *dst,
+    const char *src,
     U032 n
 )
 {
@@ -544,11 +466,6 @@ U032 NV_API_CALL os_get_page_size(void)
 
 NvU64 NV_API_CALL os_get_page_mask(void)
 {
-    /*
-     * On FreeBSD, PAGE_MASK means (PAGE_SIZE - 1); on Linux it means the
-     * opposite, ~(PAGE_SIZE - 1); that is what this function is expected
-     * to return.
-     */
     return ~PAGE_MASK;
 }
 
@@ -562,29 +479,45 @@ U032 NV_API_CALL os_get_cpu_count(void)
     return mp_ncpus;
 }
 
-RM_STATUS NV_API_CALL os_flush_cpu_cache(void)
+#ifdef NV_USE_WBINVD
+static void wbinvd_action_func(void *arg)
 {
-    /*
-     * XXX This will do for now, but this may need to be extended
-     * to make IPI calls (flushing all caches).
-     */
     __asm__ __volatile__("wbinvd": : :"memory");
-    return RM_OK;
 }
 
-void NV_API_CALL os_flush_cpu_write_combine_buffer(void)
+RM_STATUS NV_API_CALL os_flush_cpu_cache(void)
+{
+    smp_rendezvous(NULL, wbinvd_action_func, NULL, NULL);
+    wbinvd_action_func(NULL);
+    return RM_OK;
+}
+#else
+RM_STATUS NV_API_CALL os_flush_cpu_cache(void)
+{
+    pmap_invalidate_cache();
+    return RM_OK;
+}
+#endif
+
+static void sfence_action_func(void *arg)
 {
     __asm__ __volatile__("sfence": : :"memory");
 }
 
+void NV_API_CALL os_flush_cpu_write_combine_buffer(void)
+{
+    smp_rendezvous(NULL, sfence_action_func, NULL, NULL);
+    sfence_action_func(NULL);
+}
+
 RM_STATUS NV_API_CALL os_raise_smp_barrier(void)
 {
-    return RM_OK;
+    return RM_ERR_NOT_SUPPORTED;
 }
 
 RM_STATUS NV_API_CALL os_clear_smp_barrier(void)
 {
-    return RM_OK;
+    return RM_ERR_NOT_SUPPORTED;
 }
 
 struct os_mutex {
@@ -616,7 +549,7 @@ RM_STATUS NV_API_CALL os_alloc_sema(void **semaphore)
     mtx->sp = sp;
     mtx->refcnt = 1;
 
-    *semaphore = (void *) mtx;
+    *semaphore = (void *)mtx;
 
     return RM_OK;
 }
@@ -652,21 +585,21 @@ RM_STATUS NV_API_CALL os_acquire_sema(void *semaphore)
     return RM_OK;
 }
 
-BOOL NV_API_CALL os_cond_acquire_sema(void *semaphore)
+RM_STATUS NV_API_CALL os_cond_acquire_sema(void *semaphore)
 {
     struct os_mutex *mtx = semaphore;
 
     mtx_lock(&mtx->mutex_mtx);
     if (mtx->refcnt < 1) {
         mtx_unlock(&mtx->mutex_mtx);
-        return FALSE;
+        return RM_ERROR;
     } else {
         rm_disable_interrupts(mtx->sp);
         mtx->refcnt--;
         mtx_unlock(&mtx->mutex_mtx);
     }
 
-    return TRUE;
+    return RM_OK;
 }
 
 RM_STATUS NV_API_CALL os_release_sema(void *semaphore)
@@ -692,55 +625,7 @@ BOOL NV_API_CALL os_is_acquired_sema(void *semaphore)
 
 BOOL NV_API_CALL os_pat_supported(void)
 {
-    /*
-     * FreeBSD has no native PAT support and there's no good
-     * way to implement it privately as we do on Linux.
-     */
-    return FALSE;
-}
-
-void* NV_API_CALL NV_STACKWATCH_CALLBACK os_get_stack_start(void *stack_pointer)
-{
-    struct thread *td;
-#if defined(NVCPU_X86_64)
-    __asm __volatile__("movq %%gs:0,%0" : "=r" (td));
-#elif defined(NVCPU_X86)
-    __asm __volatile__("movl %%fs:0,%0" : "=r" (td));
-#endif
-    return (void *)cpu_getstack(td);
-}
-
-NvU64 NV_API_CALL os_get_current_pdpte(U032 address)
-{
-    return 0;
-}
-
-RM_STATUS NV_API_CALL os_set_mlock_capability()
-{
-    return (RM_ERROR);
-}
-
-S032 NV_API_CALL os_mlock_user_memory(
-    void *address,
-    U032 length
-)
-{
-    return -1;
-}
-
-S032 NV_API_CALL os_munlock_user_memory(
-    void *address,
-    U032 length
-)
-{
-    return -1;
-}
-
-RM_STATUS NV_API_CALL os_check_process_map_limit(
-    NvU64 proc_max_map_count
-)
-{
-    return (RM_ERROR);
+    return TRUE;
 }
 
 void NV_API_CALL os_register_compatible_ioctl(
@@ -767,3 +652,28 @@ RM_STATUS NV_API_CALL os_enable_console_access(void)
     return RM_OK;
 }
 
+NvU64 NV_API_CALL os_acquire_spinlock(void *pSema)
+{
+    struct os_mutex *mtx = pSema;
+
+    mtx_lock(&mtx->mutex_mtx);
+
+    return 0;
+}
+
+void NV_API_CALL os_release_spinlock(void *pSema, NvU64 oldIrql)
+{
+    struct os_mutex *mtx = pSema;
+
+    mtx_unlock(&mtx->mutex_mtx);
+}
+
+RM_STATUS NV_API_CALL os_get_address_space_info(
+    NvU64 *userStartAddress,
+    NvU64 *userEndAddress,
+    NvU64 *kernelStartAddress,
+    NvU64 *kernelEndAddress
+)
+{
+    return RM_ERR_NOT_SUPPORTED;
+}
