@@ -13,9 +13,19 @@
 #include "nv.h"
 #include "nv-freebsd.h"
 
-static int nvidia_pci_probe  (device_t);
-static int nvidia_pci_attach (device_t);
-static int nvidia_pci_detach (device_t);
+static void nvidia_pci_identify (driver_t *, device_t);
+static int  nvidia_pci_probe    (device_t);
+static int  nvidia_pci_attach   (device_t);
+static int  nvidia_pci_detach   (device_t);
+
+static void nvidia_pci_disable_serr  (device_t dev);
+static void nvidia_pci_enable_intx   (device_t dev);
+
+void nvidia_pci_identify(driver_t *driver, device_t parent)
+{
+    if (device_find_child(parent, "nvidia", -1) == NULL)
+        device_add_child(parent, "nvidia", -1);
+}
 
 int nvidia_pci_probe(device_t dev)
 {
@@ -58,14 +68,11 @@ int nvidia_pci_setup_intr(device_t dev)
 
     sc = device_get_softc(dev);
 
-    /* XXX Revisit! (INTR_FAST, INTR_MPSAFE) */
-    flags = INTR_TYPE_AV;
+    /* XXX Revisit! (INTR_FAST) */
+    flags = (INTR_TYPE_AV | INTR_MPSAFE);
 
-#if __FreeBSD_version >= 700031
-    status = bus_setup_intr(dev, sc->irq, flags, NULL, nvidia_intr, sc, &sc->irq_ih);
-#else
-    status = bus_setup_intr(dev, sc->irq, flags, nvidia_intr, sc, &sc->irq_ih);
-#endif
+    status = bus_setup_intr(dev, sc->irq, flags, NULL, nvidia_intr, sc,
+            &sc->irq_ih);
     if (status) {
         device_printf(dev, "NVRM: HW ISR setup failed.\n");
         goto fail;
@@ -92,6 +99,20 @@ fail:
     return status;
 }
 
+static void nvidia_pci_disable_serr(device_t dev)
+{
+    U016 word;
+    word = (os_pci_read_word(dev, PCIR_COMMAND) & ~PCIM_CMD_SERRESPEN);
+    os_pci_write_word(dev, PCIR_COMMAND, word);
+}
+
+static void nvidia_pci_enable_intx(device_t dev)
+{
+    U016 word;
+    word = (os_pci_read_word(dev, PCIR_COMMAND) & ~PCIM_CMD_INTXDIS);
+    os_pci_write_word(dev, PCIR_COMMAND, word);
+}
+
 void nvidia_pci_check_config_space(
     nv_stack_t *sp,
     device_t dev,
@@ -119,6 +140,12 @@ void nvidia_pci_check_config_space(
     if ((word & PCIM_CMD_MEMEN) == 0)
         pci_enable_io(dev, SYS_RES_MEMORY);
 
+    if ((word & PCIM_CMD_SERRESPEN) != 0)
+        nvidia_pci_disable_serr(dev);
+
+    if ((word & PCIM_CMD_INTXDIS) != 0)
+        nvidia_pci_enable_intx(dev);
+
     if (check_the_bars)
     {
         for (i = 0; i < NV_GPU_NUM_BARS; i++) {
@@ -191,9 +218,7 @@ int nvidia_pci_attach(device_t dev)
     }
 
     sc = device_get_softc(dev); /* first reference */
-
     bzero(sc, sizeof(nvidia_softc_t));
-    STAILQ_INIT(&sc->filep_queue);
 
     sc->nv_state = malloc(sizeof(nv_state_t), M_NVIDIA, M_WAITOK | M_ZERO);
     if (sc->nv_state == NULL)
@@ -267,7 +292,7 @@ int nvidia_pci_attach(device_t dev)
         goto fail;
     }
 
-    mtx_init(&sc->rm_mtx, "dev.rm_mtx", NULL, MTX_SPIN | MTX_RECURSE);
+    callout_init(&sc->timer, CALLOUT_MPSAFE);
     sx_init(&sc->api_sx, "dev.api_sx");
 
     return 0;
@@ -306,8 +331,6 @@ int nvidia_pci_detach(device_t dev)
     }
 
     nv_unlock_api(nv);
-
-    mtx_destroy(&sc->rm_mtx);
     sx_destroy(&sc->api_sx);
 
     status = nvidia_pci_teardown_intr(dev);
@@ -336,12 +359,13 @@ fail:
 }
 
 static device_method_t nvidia_pci_methods[] = {
-    DEVMETHOD( device_probe,   nvidia_pci_probe  ),
-    DEVMETHOD( device_attach,  nvidia_pci_attach ),
-    DEVMETHOD( device_detach,  nvidia_pci_detach ),
+    DEVMETHOD( device_identify, nvidia_pci_identify ),
+    DEVMETHOD( device_probe,    nvidia_pci_probe    ),
+    DEVMETHOD( device_attach,   nvidia_pci_attach   ),
+    DEVMETHOD( device_detach,   nvidia_pci_detach   ),
 #ifdef NV_SUPPORT_ACPI_PM
-    DEVMETHOD( device_suspend, nvidia_suspend    ),
-    DEVMETHOD( device_resume,  nvidia_resume     ),
+    DEVMETHOD( device_suspend,  nvidia_suspend      ),
+    DEVMETHOD( device_resume,   nvidia_resume       ),
 #endif
     { 0, 0 }
 };
@@ -352,7 +376,7 @@ static driver_t nvidia_pci_driver = {
     sizeof(struct nvidia_softc)
 };
 
-DRIVER_MODULE(nvidia, pci, nvidia_pci_driver, nvidia_devclass, nvidia_modevent, 0);
+DRIVER_MODULE(nvidia, vgapci, nvidia_pci_driver, nvidia_devclass, nvidia_modevent, 0);
 
 MODULE_DEPEND(nvidia, mem, 1, 1, 1);
 MODULE_DEPEND(nvidia, io, 1, 1, 1);
@@ -364,4 +388,3 @@ MODULE_DEPEND(nvidia, agp, 1, 1, 1);
 #ifdef NV_SUPPORT_LINUX_COMPAT /* (COMPAT_LINUX || COMPAT_LINUX32) */
 MODULE_DEPEND(nvidia, linux, 1, 1, 1);
 #endif
-
