@@ -59,37 +59,6 @@ void nvidia_sysctl_init(void)
             0,
             "NVIDIA Resource Manager (NVRM) Version");
 
-#if defined(NV_STACKWATCH)
-    do {
-        oid = SYSCTL_ADD_NODE(&sysctl_ctx,
-                SYSCTL_CHILDREN(oid_nvidia),
-                OID_AUTO,
-                "stackwatch",
-                CTLFLAG_RD | CTLFLAG_DYN,
-                0,
-                "NVIDIA StackWatch Statistics");
-
-        SYSCTL_ADD_PROC(&sysctl_ctx,
-                SYSCTL_CHILDREN(oid),
-                OID_AUTO,
-                "control",
-                CTLTYPE_INT | CTLFLAG_WR,
-                NULL, 0,
-                nvidia_sysctl_stackwatch,
-                "I",
-                "NVIDIA StackWatch Control");
-
-        SYSCTL_ADD_STRUCT(&sysctl_ctx,
-                SYSCTL_CHILDREN(oid),
-                OID_AUTO,
-                "stats",
-                CTLFLAG_RD | CTLFLAG_DYN,
-                &stk_stats,
-                stk_stats,
-                "NVIDIA StackWatch Statistics");
-    } while (0);
-#endif
-
     oid_registry = SYSCTL_ADD_NODE(&sysctl_ctx,
             SYSCTL_CHILDREN(oid_nvidia),
             OID_AUTO,
@@ -111,7 +80,7 @@ void nvidia_sysctl_init(void)
         entry++;
     } while(entry->name != NULL);
 
-    option_string = malloc(1, M_NVIDIA, M_WAITOK | M_ZERO);
+    option_string = malloc(1, M_NVIDIA, (M_WAITOK | M_ZERO));
 
     SYSCTL_ADD_PROC(&sysctl_ctx,
             SYSCTL_CHILDREN(oid_registry),
@@ -235,7 +204,21 @@ static void* nvidia_find_bridge(void)
                 dev = pci_find_bsf(bus, device, func);
 
                 if (dev == NULL)
-                    break;
+                {
+                    if (func == 0) {
+                        /*
+                         * If a read to function zero of a specified
+                         * bus/device master aborts, then it is assumed
+                         * that no such device exists on the bus since
+                         * devices are required to implement function
+                         * number zero.
+                         * In this case reads to the remaining functions
+                         * are not necessary.
+                         */
+                        break;
+                    } else
+                        continue;
+                }
 
                 if (pci_get_class(dev) != PCIC_BRIDGE)
                     break;
@@ -300,7 +283,7 @@ int nvidia_sysctl_dev_vbios(SYSCTL_HANDLER_ARGS)
                 &vbios[3], &vbios[4]) != RM_OK) {
         /*
          * The VBIOS version is only accessible after the device has been
-         * initialized with rm_init_adapter. 
+         * initialized with rm_init_adapter.
          */
         sprintf(vbios_version, "??.??.??.??.??");
     } else {
@@ -451,7 +434,7 @@ int nvidia_sysctl_agp_rate_status(SYSCTL_HANDLER_ARGS)
 {
     device_t dev = arg1;
     U008 agp_rate[16];
-    
+
     U032 tmp = nvidia_sys_agp_cmd(dev);
 
     if ((tmp & 0x100) && (tmp &= 0x07)) {
@@ -509,36 +492,6 @@ int nvidia_sysctl_agp_status(SYSCTL_HANDLER_ARGS)
     return SYSCTL_OUT(req, agp_cmd, strlen(agp_cmd) + 1);
 }
 
-#if defined(NV_STACKWATCH)
-int nvidia_sysctl_stackwatch(SYSCTL_HANDLER_ARGS)
-{
-    int error, cmd;
-
-    if (req->newptr == NULL || req->newlen != sizeof(int))
-        return -EINVAL;
-
-    error = SYSCTL_IN(req, &cmd, sizeof(int));
-    if (error)
-        return error;
-
-    /*
-     * This is used by stprof to stop data acquisition prior to
-     * querying the current profile and to reset/re-enable it
-     * afterwards.
-     */
-    switch (cmd) {
-        case 0:
-            nv_stackwatch_freeze(); break;
-        case 1:
-            nv_stackwatch_reset();  break;
-        default:
-            return -EINVAL;
-    }
-
-    return 0; /* we don't return anything */
-}
-#endif
-
 void nv_sysctl_init(nv_state_t *nv)
 {
     struct sysctl_oid *oid;
@@ -730,4 +683,3 @@ void nv_sysctl_exit(nv_state_t *nv)
     struct nvidia_softc *sc = nv->os_state;
     sysctl_ctx_free(&sc->sysctl_ctx);
 }
-
