@@ -50,6 +50,12 @@
 
 #ifndef NVTYPES_INCLUDED
 #define NVTYPES_INCLUDED
+
+/* XAPIGEN - this file is not suitable for (nor needed by) xapigen.         */
+/*           Rather than #ifdef out every such include in every sdk         */
+/*           file, punt here.                                               */
+#if !defined(XAPIGEN)        /* rest of file */
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -74,6 +80,20 @@ typedef signed short       NvS16; /* -32768 to 32767                         */
 typedef float              NvF32; /* IEEE Single Precision (S1E8M23)         */
 typedef double             NvF64; /* IEEE Double Precision (S1E11M52)        */
 
+// Macros to get the MSB and LSB of a 16 bit unsigned number
+#define NvU16_HI08(n) ((NvU8)(((NvU16)(n)) >> 8))
+#define NvU16_LO08(n) ((NvU8)((NvU16)(n)))
+
+// Macro to build a NvU16 from msb and lsb bytes. 
+#define NvU16_BUILD(msb, lsb)  (((msb) << 8)|(lsb))
+
+#if defined(macosx) || defined(MACOS) || defined(NV_MACINTOSH) || \
+    defined(NV_MACINTOSH_64)
+typedef char*              NVREGSTR;
+#else
+typedef NvU8*              NVREGSTR;
+#endif
+
 /* mainly for 64-bit linux, where long is 64 bits
  * and win9x, where int is 16 bit.
  */
@@ -97,7 +117,7 @@ typedef signed int         NvS32; /* -2147483648 to 2147483647               */
 #endif
 
 /* 64-bit types for compilers that support them, plus some obsolete variants */
-#if defined(__GNUC__) || defined(__arm) || defined(__IAR_SYSTEMS_ICC__) || defined(__ghs__) || defined(_WIN64)
+#if defined(__GNUC__) || defined(__arm) || defined(__IAR_SYSTEMS_ICC__) || defined(__ghs__) || defined(_WIN64) || defined(__SUNPRO_C) || defined(__SUNPRO_CC)
 typedef unsigned long long NvU64; /* 0 to 18446744073709551615          */
 typedef          long long NvS64; /* 2^-63 to 2^63-1                    */
 #else
@@ -111,10 +131,10 @@ typedef NvU8 NvBool;
 
 /* Macros to extract the low and high parts of a 64-bit unsigned integer */
 /* Also designed to work if someone happens to pass in a 32-bit integer */
-#define NvU64_HI32(n) ((NvU32)(((NvU64)(n)) >> 32))
-#define NvU64_LO32(n) ((NvU32)((NvU64)(n)))
-#define NvU40_HI32(n) ((NvU32)((((NvU64)(n))>>8)&0xffffffff))
-#define NvU40_HI24of32(n) ((NvU32)((((NvU64)(n)& 0xffffffff))&0xffffff00))
+#define NvU64_HI32(n)     ((NvU32)((((NvU64)(n)) >> 32) & 0xffffffff))
+#define NvU64_LO32(n)     ((NvU32)(( (NvU64)(n))        & 0xffffffff))
+#define NvU40_HI32(n)     ((NvU32)((((NvU64)(n)) >>  8) & 0xffffffff))
+#define NvU40_HI24of32(n) ((NvU32)(  (NvU64)(n)         & 0xffffff00))
 
  /***************************************************************************\
 |*                                                                           *|
@@ -126,18 +146,32 @@ typedef NvU8 NvBool;
 
 typedef void*              NvP64; /* 64 bit void pointer                     */
 typedef NvU64             NvUPtr; /* pointer sized unsigned int              */
+typedef NvS64             NvSPtr; /* pointer sized signed int                */
+typedef NvU64           NvLength; /* length to agree with sizeof             */
 
-#define NvP64_VALUE(n)    (n)
+#define NvP64_VALUE(n)        (n)
+#define NvP64_fmt "%p"
+
+#define KERNEL_POINTER_FROM_NvP64(p,v) ((p)(v))
+#define NvP64_PLUS_OFFSET(p,o) (NvP64)((NvU64)(p) + (NvU64)(o))
 
 #else
 
 typedef NvU64              NvP64; /* 64 bit void pointer                     */
 typedef NvU32             NvUPtr; /* pointer sized unsigned int              */
+typedef NvS32             NvSPtr; /* pointer sized signed int                */
+typedef NvU32           NvLength; /* length to agree with sizeof             */
 
-#define NvP64_VALUE(n)    ((void *)(NvUPtr)(n))
+#define NvP64_VALUE(n)        ((void *)(NvUPtr)(n))
+#define NvP64_fmt "0x%llx"
+
+#define KERNEL_POINTER_FROM_NvP64(p,v) ((p)(NvUPtr)(v))
+#define NvP64_PLUS_OFFSET(p,o) ((p) + (NvU64)(o))
 
 #endif
 
+#define NvP64_NULL       (NvP64)0
+
 // XXX Obsolete -- get rid of me...
 typedef NvP64 NvP64_VALUE_T;
 typedef NvUPtr NV_UINTPTR_T;
@@ -146,6 +180,7 @@ typedef NvUPtr NV_UINTPTR_T;
 
 /* Useful macro to hide required double cast */
 #define NV_PTR_TO_NvP64(n) (NvP64)(NV_UINTPTR_T)(n)
+#define KERNEL_POINTER_TO_NvP64(p) ((NvP64)(uintptr_t)(p))
 
 /* obsolete stuff  */
 /* MODS needs to be able to build without these definitions because they collide
@@ -173,6 +208,25 @@ typedef unsigned long  ULONG;
 typedef unsigned char *PUCHAR;
 #endif
 
+ /***************************************************************************\
+|*                                                                           *|
+|*  Limits for common types.                                                 *|
+|*                                                                           *|
+ \***************************************************************************/
+
+#define NV_S8_MIN       ((NvS8) -128)
+#define NV_S8_MAX       ((NvS8) +127)
+#define NV_U8_MIN       ((NvU8) +0)
+#define NV_U8_MAX       ((NvU8) +255)
+#define NV_S16_MIN      ((NvS16)-32768)
+#define NV_S16_MAX      ((NvS16)+32767)
+#define NV_U16_MIN      ((NvU16)+0)
+#define NV_U16_MAX      ((NvU16)+65535)
+#define NV_S32_MIN      ((NvS32)-2147483648)
+#define NV_S32_MAX      ((NvS32)+2147483647)
+#define NV_U32_MIN      ((NvU32)+0)
+#define NV_U32_MAX      ((NvU32)+4294967295)
+
 #if !defined(NV_PTR)
     /* Supports 32bit libraries on Win64
        See drivers\opengl\include\nvFirst.h for explanation */
@@ -181,8 +235,7 @@ typedef unsigned char *PUCHAR;
 #endif
 
 /* Aligns fields in structs  so they match up between 32 and 64 bit builds */
-/* XXX Mac still uses structures without alignment -- needs to be fixed */
-#if (defined(__GNUC__) || defined(NV_QNX)) && !defined(NV_MACINTOSH)
+#if defined(__GNUC__) || defined(NV_QNX)
 #define NV_ALIGN_BYTES(size) __attribute__ ((aligned (size)))
 #elif defined(__arm)
 #define NV_ALIGN_BYTES(size) __align(ALIGN)
@@ -205,8 +258,8 @@ typedef unsigned char *PUCHAR;
 #endif
 
 
-/***************************************************************************\
-|*                      Function Declaration Types                          *|
+ /***************************************************************************\
+|*                       Function Declaration Types                          *|
  \***************************************************************************/
 
 // stretching the meaning of "nvtypes", but this seems to least offensive
@@ -230,11 +283,11 @@ typedef unsigned char *PUCHAR;
 
     #define NV_FORCERESULTCHECK
 
-#else // defined(NV_WINDOWS)
+#else // ! defined(NV_WINDOWS)
 
     #if defined(__GNUC__) || defined(__INTEL_COMPILER)
     #define NV_INLINE __inline__
-    #elif defined (macintosh)
+    #elif defined (macintosh) || defined(__SUNPRO_C) || defined(__SUNPRO_CC)
     #define NV_INLINE inline
     #elif defined(__arm)
     #define NV_INLINE __inline
@@ -280,8 +333,14 @@ typedef unsigned char *PUCHAR;
 
 #endif  // defined(NV_WINDOWS)
 
+
+#include "xapi-sdk.h"       /* XAPIGEN sdk macros for C */
+
+
 #ifdef __cplusplus
 };
 #endif
 
+#endif /* ! XAPIGEN */
+
 #endif /* NVTYPES_INCLUDED */
