@@ -50,16 +50,6 @@
 #endif
 #endif /* !defined(NV_API_CALL) */
 
-#if defined(NV_STACKWATCH)
-#if (defined(NVCPU_X86) || defined(NVCPU_X86_64)) && defined(__use_altstack__)
-#define NV_STACKWATCH_CALLBACK __attribute__((no_instrument_function,altstack(false)))
-#else
-#define NV_STACKWATCH_CALLBACK __attribute__((no_instrument_function))
-#endif
-#else
-#define NV_STACKWATCH_CALLBACK
-#endif
-
 /*
  * ---------------------------------------------------------------------------
  *
@@ -68,76 +58,68 @@
  * ---------------------------------------------------------------------------
  */
 
-U032        NV_API_CALL  os_get_page_size            (void);
+NvU32       NV_API_CALL  os_get_page_size            (void);
 NvU64       NV_API_CALL  os_get_page_mask            (void);
 NvU64       NV_API_CALL  os_get_system_memory_size   (void);
-RM_STATUS   NV_API_CALL  os_alloc_mem                (void **, U032);
+RM_STATUS   NV_API_CALL  os_alloc_mem                (void **, NvU32);
 void        NV_API_CALL  os_free_mem                 (void *);
-RM_STATUS   NV_API_CALL  os_get_current_time         (U032 *, U032 *);
-RM_STATUS   NV_API_CALL  os_delay                    (U032);
-RM_STATUS   NV_API_CALL  os_delay_us                 (U032);
-U032        NV_API_CALL  os_get_cpu_frequency        (void);
-U032        NV_API_CALL  os_get_current_process      (void);
-RM_STATUS   NV_API_CALL  os_kill_process             (U032, U032);
-U008*       NV_API_CALL  os_string_copy              (U008 *, const U008 *);
-RM_STATUS   NV_API_CALL  os_strncpy_from_user        (U008 *, const U008 *, U032);
-S032        NV_API_CALL  os_string_compare           (const U008 *, const U008 *);
-U032        NV_API_CALL  os_string_length            (const U008 *);
-U008*       NV_API_CALL  os_mem_copy                 (U008 *, const U008 *, U032);
-RM_STATUS   NV_API_CALL  os_memcpy_from_user         (void *, const void *, U032);
-RM_STATUS   NV_API_CALL  os_memcpy_to_user           (void *, const void *, U032);
-void*       NV_API_CALL  os_mem_set                  (void *, U008, U032);
-S032        NV_API_CALL  os_mem_cmp                  (const U008 *, const U008 *, U032);
-void*       NV_API_CALL  os_pci_init_handle          (U008, U008, U008, U016 *, U016 *);
-U008        NV_API_CALL  os_pci_read_byte            (void *, U008);
-U016        NV_API_CALL  os_pci_read_word            (void *, U008);
-U032        NV_API_CALL  os_pci_read_dword           (void *, U008);
-void        NV_API_CALL  os_pci_write_byte           (void *, U008, U008);
-void        NV_API_CALL  os_pci_write_word           (void *, U008, U016);
-void        NV_API_CALL  os_pci_write_dword          (void *, U008, U032);
-void*       NV_API_CALL  os_map_kernel_space         (NvU64, NvU64, U032);
+RM_STATUS   NV_API_CALL  os_get_current_time         (NvU32 *, NvU32 *);
+RM_STATUS   NV_API_CALL  os_delay                    (NvU32);
+RM_STATUS   NV_API_CALL  os_delay_us                 (NvU32);
+NvU32       NV_API_CALL  os_get_cpu_frequency        (void);
+NvU32       NV_API_CALL  os_get_current_process      (void);
+RM_STATUS   NV_API_CALL  os_get_current_thread       (NvU64 *);
+char*       NV_API_CALL  os_string_copy              (char *, const char *);
+RM_STATUS   NV_API_CALL  os_strncpy_from_user        (char *, const char *, NvU32);
+NvU32       NV_API_CALL  os_string_length            (const char *);
+NvU8*       NV_API_CALL  os_mem_copy                 (NvU8 *, const NvU8 *, NvU32);
+RM_STATUS   NV_API_CALL  os_memcpy_from_user         (void *, const void *, NvU32);
+RM_STATUS   NV_API_CALL  os_memcpy_to_user           (void *, const void *, NvU32);
+void*       NV_API_CALL  os_mem_set                  (void *, NvU8, NvU32);
+NvS32       NV_API_CALL  os_mem_cmp                  (const NvU8 *, const NvU8 *, NvU32);
+void*       NV_API_CALL  os_pci_init_handle          (NvU8, NvU8, NvU8, NvU16 *, NvU16 *);
+NvU8        NV_API_CALL  os_pci_read_byte            (void *, NvU8);
+NvU16       NV_API_CALL  os_pci_read_word            (void *, NvU8);
+NvU32       NV_API_CALL  os_pci_read_dword           (void *, NvU8);
+void        NV_API_CALL  os_pci_write_byte           (void *, NvU8, NvU8);
+void        NV_API_CALL  os_pci_write_word           (void *, NvU8, NvU16);
+void        NV_API_CALL  os_pci_write_dword          (void *, NvU8, NvU32);
+void*       NV_API_CALL  os_map_kernel_space         (NvU64, NvU64, NvU32);
 void        NV_API_CALL  os_unmap_kernel_space       (void *, NvU64);
 RM_STATUS   NV_API_CALL  os_flush_cpu_cache          (void);
 void        NV_API_CALL  os_flush_cpu_write_combine_buffer(void);
-RM_STATUS   NV_API_CALL  os_set_mem_range            (U032, U032, U032);
-RM_STATUS   NV_API_CALL  os_unset_mem_range          (U032, U032);
-BOOL        NV_API_CALL  os_pci_device_present       (U016, U016);
-U008        NV_API_CALL  os_io_read_byte             (PHWINFO, U032);
-U016        NV_API_CALL  os_io_read_word             (PHWINFO, U032);
-U032        NV_API_CALL  os_io_read_dword            (PHWINFO, U032);
-void        NV_API_CALL  os_io_write_byte            (PHWINFO, U032, U008);
-void        NV_API_CALL  os_io_write_word            (PHWINFO, U032, U016);
-void        NV_API_CALL  os_io_write_dword           (PHWINFO, U032, U032);
+RM_STATUS   NV_API_CALL  os_set_mem_range            (NvU64, NvU64, NvU32);
+RM_STATUS   NV_API_CALL  os_unset_mem_range          (NvU64, NvU64);
+BOOL        NV_API_CALL  os_pci_device_present       (NvU16, NvU16);
+NvU8        NV_API_CALL  os_io_read_byte             (NvU32);
+NvU16       NV_API_CALL  os_io_read_word             (NvU32);
+NvU32       NV_API_CALL  os_io_read_dword            (NvU32);
+void        NV_API_CALL  os_io_write_byte            (NvU32, NvU8);
+void        NV_API_CALL  os_io_write_word            (NvU32, NvU16);
+void        NV_API_CALL  os_io_write_dword           (NvU32, NvU32);
 BOOL        NV_API_CALL  os_is_administrator         (PHWINFO);
 void        NV_API_CALL  os_dbg_init                 (void);
-void        NV_API_CALL NV_STACKWATCH_CALLBACK os_dbg_breakpoint  (void);
-void        NV_API_CALL  os_dbg_set_level            (U032);
-NvU64       NV_API_CALL  os_get_current_pdpte        (U032);
-U032        NV_API_CALL  os_get_cpu_count            (void);
+void        NV_API_CALL  os_dbg_breakpoint           (void);
+void        NV_API_CALL  os_dbg_set_level            (NvU32);
+NvU32       NV_API_CALL  os_get_cpu_count            (void);
 RM_STATUS   NV_API_CALL  os_raise_smp_barrier        (void);
 RM_STATUS   NV_API_CALL  os_clear_smp_barrier        (void);
 RM_STATUS   NV_API_CALL  os_disable_console_access   (void);
 RM_STATUS   NV_API_CALL  os_enable_console_access    (void);
 RM_STATUS   NV_API_CALL  os_registry_init            (void);
-
-void*       NV_API_CALL  os_map_kernel_space_high    (U032, U032);
-void        NV_API_CALL  os_unmap_kernel_space_high  (void *, U032, U032);
-RM_STATUS   NV_API_CALL  os_set_mlock_capability     (void);
-RM_STATUS   NV_API_CALL  os_check_process_map_limit  (NvU64);
-S032        NV_API_CALL  os_mlock_user_memory        (void *, U032);
-S032        NV_API_CALL  os_munlock_user_memory      (void *, U032);
-
-void*       NV_API_CALL NV_STACKWATCH_CALLBACK os_get_stack_start  (void *);
-
 RM_STATUS   NV_API_CALL  os_alloc_sema               (void **);
 RM_STATUS   NV_API_CALL  os_free_sema                (void *);
 RM_STATUS   NV_API_CALL  os_acquire_sema             (void *);
-BOOL        NV_API_CALL  os_cond_acquire_sema        (void *);
+RM_STATUS   NV_API_CALL  os_cond_acquire_sema        (void *);
 RM_STATUS   NV_API_CALL  os_release_sema             (void *);
 BOOL        NV_API_CALL  os_is_acquired_sema         (void *);
+RM_STATUS   NV_API_CALL  os_schedule                 (void);
+NvU64       NV_API_CALL  os_acquire_spinlock         (void *);
+void        NV_API_CALL  os_release_spinlock         (void *, NvU64);
+RM_STATUS   NV_API_CALL  os_get_address_space_info   (NvU64 *, NvU64 *, NvU64 *, NvU64 *);
 
-void        NV_API_CALL  os_register_compatible_ioctl    (U032, U032);
-void        NV_API_CALL  os_unregister_compatible_ioctl  (U032, U032);
+void        NV_API_CALL  os_register_compatible_ioctl    (NvU32, NvU32);
+void        NV_API_CALL  os_unregister_compatible_ioctl  (NvU32, NvU32);
 
 /* BOOL os_pat_supported(void) 
  * report to core resman whether pat is supported for marking cache attributes
@@ -182,8 +164,8 @@ void        NV_API_CALL  os_dump_stack (void);
 
 
 void NV_API_CALL  out_string(const char *str);
-int  NV_API_CALL  nv_printf(U032 debuglevel, const char *printf_format, ...);
-void NV_API_CALL  nv_prints(U032 debuglevel, const char *string);
+int  NV_API_CALL  nv_printf(NvU32 debuglevel, const char *printf_format, ...);
+void NV_API_CALL  nv_prints(NvU32 debuglevel, const char *string);
 int  NV_API_CALL  nv_snprintf(char *buf, unsigned int size, const char *fmt, ...);
 void NV_API_CALL  nv_os_log(int log_level, const char *fmt, void *ap);
 
