@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 1993-2002 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 1993-2008 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -84,7 +84,24 @@ typedef NvU32 RM_STATUS;
 #define RM_ERR_TIMEOUT                  0x00000038
 #define RM_ERR_BUFFER_TOO_SMALL         0x00000039
 #define RM_ERR_NO_SUCH_DOMAIN           0x0000003A
+#define RM_ERR_I2C_ERROR                0x0000003B
+#define RM_ERR_FREQ_NOT_SUPPORTED       0x0000003C
+#define RM_ERR_INVALID_REQUEST          0x0000003D
+#define RM_ERR_MORE_PROCESSING_REQUIRED 0x0000003E
+#define RM_ERR_MISSING_TABLE_ENTRY      0x0000003F
+#define RM_ERR_NO_INTR_PENDING          0x00000040
+#define RM_ERR_INSUFFICIENT_PERMISSIONS 0x00000041
+#define RM_ERR_TIMEOUT_RETRY            0x00000042
+#define RM_ERR_NOT_READY                0x00000043
+#define RM_ERR_BROKEN_FB                0x00000044
+#define RM_ERR_GPU_IS_LOST              0x00000045
+#define RM_ERR_GPU_IN_FULLCHIP_RESET    0x00000046
+#define RM_ERR_REJECTED_VBIOS           0x00000047
+#define RM_ERR_NOT_ENOUGH_POWER         0x00000048
+#define RM_ERR_I2C_SPEED_TOO_HIGH       0X00000049
 
+// Warnings:
 #define RM_WARN_NULL_OBJECT             0x00000100
+#define RM_WARN_INCORRECT_PERFMON_DATA  0x00000101
 
 #endif /* _RMRETVAL_H_ */
